unit wavU;

//      :
// 44100 Hz, 16bit, mono

interface

uses
  Classes, SysUtils, Dialogs;

type

  TIntArray = array of integer;

  //  WAV
  TWaveHeader = record
    idRIFF: array [0..3] of Char; //'RIFF'
    RiffLen: Longint; // 8
    idWave: array [0..3] of Char; //'WAVE'
    idFmt: array [0..3] of Char; //'fmt '
    InfoLen: Longint; //
    WaveType: smallint; // 1 - PCM
    Channels: smallint; // 1 -  
    SamplesPerSec: Longint; // 44100 -   
    BytesPerSecond: Longint; // 88200 -   
    BytesPerSample: smallint; // 2 -   
    BitsPerSample: smallint; // 16 -   
  end;

  //  DATA ( WAV)
  TDataHeader = record
    idData: array [0..3] of Char; //'data'
    DataLen: Longint; //      
  end;

  // - ..)
  //       
  TWave = record
    Header: TWaveHeader; // WAV-
    DataHeader: TDataHeader; // DATA-
    SamplesCount: integer; //   
    LengthSec: real; //   
    DataSeek: int64; //    
    Filesize: int64;
  end;

function mix(a, b: integer): integer;

//     
procedure ReadWave(Filename: string; var Wave: TWave);
procedure CreateWave(var Wave: Twave; Len: Longint);

var
  dev: string = ' //'; //       

implementation

function mix(a, b: integer): integer;
begin
  Result := a;
  if a < b then
    Result := b;
end;

//  wav-
procedure ReadWave(Filename: string; var Wave: TWave);
var
  F: TFileStream;
begin
  try
    //  :
    F := TFileStream.Create(Filename, fmShareDenyNone);
    //  :
    F.Read(Wave.Header, sizeof(TWaveHeader));
    with Wave.Header do
    begin
      if idRiff <> 'RIFF' then
        raise EReadError.Create('Wrong idRIFF');
      if idWave <> 'WAVE' then
        raise EReadError.Create('Wrong idWAVE');
      if idFmt <> 'fmt ' then
        raise EReadError.Create('Wrong idFmt');
      if WaveType <> 1 then
        raise EReadError.Create('Unknown format');
      F.Seek(InfoLen - 16, soFromCurrent);
    end;
    F.Read(Wave.DataHeader, sizeof(TDataHeader));
    if Wave.DataHeader.idData = 'fact' then
    begin
      F.Seek(4, soFromCurrent);
      F.Read(Wave.DataHeader, sizeof(TDataHeader));
    end;
    if Wave.DataHeader.idData <> 'data' then
      raise EReadError.Create('Wrong idData');
    Wave.SamplesCount := Wave.DataHeader.DataLen div
      (Wave.Header.Channels * Wave.Header.BitsPerSample div 8);
    Wave.LengthSec := Wave.SamplesCount / Wave.Header.SamplesPerSec;
    Wave.DataSeek := F.Position;
    Wave.Filesize := F.Size;
    //  
    F.Free;
  except
    raise Exception.Create('Problems with file reading');
  end;
end;

//   
procedure CreateWave(var Wave: Twave; Len: Longint);
begin
  Wave.DataHeader.idData := 'data';
  Wave.DataHeader.DataLen := Len;
  Wave.Filesize := SizeOf(TWaveHeader) + SizeOf(TDataHeader) + Len;
  with Wave.Header do
  begin
    idRIFF := 'RIFF';
    RiffLen := Wave.Filesize - 8;
    idWave := 'WAVE';
    idFmt := 'fmt ';
    InfoLen := 16;
    WaveType := 1;
    Channels := 1;
    SamplesPerSec := 44100;
    BitsPerSample := 16;
    BytesPerSample := BitsPerSample div 8;
    BytesPerSecond := SamplesPerSec * BytesPerSample;
  end;
  Wave.DataSeek := SizeOf(TWaveHeader) + SizeOf(TDataHeader);
  Wave.SamplesCount := Wave.DataHeader.DataLen div
      (Wave.Header.Channels * Wave.Header.BitsPerSample div 8);
  Wave.LengthSec := Wave.SamplesCount / Wave.Header.SamplesPerSec;
end;

end.
